# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

"""
Flask application
"""

import datetime
import logging
import os

import flask
import flask_sqlalchemy
import flask_restless

import ccsdb


logging.basicConfig()

_log = logging.getLogger(__name__)

_app = flask.Flask(__name__)

_app.config.from_object('ccsdb.default_config')
if 'CCSDB_CONFIG' in os.environ:  # pragma: no cover
    _app.config.from_envvar('CCSDB_CONFIG')

_db = flask_sqlalchemy.SQLAlchemy(_app)


# DB MODEL

class Status(_db.Model):
    """ This table is the main table of the application, it stores the CI
    status for a given git commit.
    """
    __tablename__ = 'status'

    id = _db.Column(_db.Integer, unique=True)
    commit = _db.Column(_db.String(40), primary_key=True)
    project = _db.Column(_db.Text, nullable=False)
    namespace = _db.Column(_db.Text, nullable=False)
    status = _db.Column(_db.Text, nullable=False)

    date_created = _db.Column(
        _db.DateTime, nullable=False,
        default=datetime.datetime.utcnow)
    date_updated = _db.Column(
        _db.DateTime, nullable=False,
        default=datetime.datetime.utcnow,
        onupdate=_db.func.now())

    @classmethod
    def get(self, commit):
        """ Return the status corresponding to the specified commit. """
        return Status.query.filter(commit == commit).first()


class Message(_db.Model):
    __tablename__ = 'messages'

    id = _db.Column(_db.Integer, primary_key=True)
    commit = _db.Column(
        _db.String(40),
        _db.ForeignKey(
            'status.commit', ondelete="CASCADE", onupdate="CASCADE")
    )
    message = _db.Column(_db.Text, nullable=False)

    date_created = _db.Column(
        _db.DateTime, nullable=False,
        default=datetime.datetime.utcnow)
    date_updated = _db.Column(
        _db.DateTime, nullable=False,
        default=datetime.datetime.utcnow,
        onupdate=_db.func.now())


# Basic welcome page

@_app.route('/')
def index():
    """ Front page of the application.
    """
    return flask.render_template(
        'index.html', version=ccsdb.__version__)


# API app

apimanager = flask_restless.APIManager(_app, flask_sqlalchemy_db=_db)

apimanager.create_api(Status, methods=['GET'])
apimanager.create_api(Message, methods=['GET'], primary_key='commit')
